@@ -147,6 +147,10 @@ def writeMocapFile(writer, fileName, data, copyFromFile=None, reader=None):
 
     """
 
+    markerUnit = 'mm'
+    angleUnit = 'deg'
+    translationUnit = 'mm'
+    lengthUnit = 'mm'
     if copyFromFile is not None:
         # Open C3D pointer
         reader.readFromFile(copyFromFile)
@@ -164,7 +168,7 @@ def writeMocapFile(writer, fileName, data, copyFromFile=None, reader=None):
                 if 'framesNumber' in data[section]:
                     nVectorFrames = data[section]['framesNumber']
                 else:
-                    firstVectorName = data[section]['data'].keys()[0]
+                    firstVectorName = list(data[section]['data'].keys())[0]
                     firstVector = data[section]['data'][firstVectorName]
                     nVectorFrames = firstVector.shape[0]
                 writer.setNVectorFrames(nVectorFrames)
@@ -179,7 +183,7 @@ def writeMocapFile(writer, fileName, data, copyFromFile=None, reader=None):
                 writer.setVectorFrequency(vectorFreq)
 
     for section in data:
-        if section in ['markers', 'angles']:
+        if section in ['markers', 'angles', 'translations', 'lengths']:
             vecType = section[:-1]
             # Write vector data
             vectors = data[section]['data']
@@ -198,6 +202,16 @@ def writeMocapFile(writer, fileName, data, copyFromFile=None, reader=None):
                         vectorData = vectors[v].copy()
                     else:
                         raise Exception('not solvable conflict between existing and new angles unit')
+                elif section == 'translations':
+                    if translationUnit == newVectorUnit:
+                        vectorData = vectors[v].copy()
+                    else:
+                        raise Exception('not solvable conflict between existing and new translations unit')
+                elif section == 'lengths':
+                    if lengthUnit == newVectorUnit:
+                        vectorData = vectors[v].copy()
+                    else:
+                        raise Exception('not solvable conflict between existing and new lengths unit')
                 writer.addVector(v, vectorData, vecType)
 
     # Write to C3D
