@@ -6,7 +6,7 @@
 
 import numpy as np
 
-from . import kine, kine_or, ligaments as liga
+from . import kine, kine_or, ligaments as liga, fio, vtkh
 
 import os
 from mpl_toolkits.mplot3d import Axes3D
@@ -82,6 +82,8 @@ def performAnalysis(
     }
 
     axes = {}
+    
+    meshes = {}
 
     for segName in segList:
 
@@ -134,6 +136,10 @@ def performAnalysis(
                 axes[axesName] = [[]] * N
                 for i in range(N):
                     axes[axesName][i] = np.array([axesPointsGlob['O'][i,:], axesPointsGlob[a][i,:]])
+        
+        geometryFile = defSegPose['geometryFile']
+        if geometryFile != '':
+            meshes[segName] = fio.readSTL(geometryFile)
             
     results['poses'] = poses
         
@@ -266,32 +272,102 @@ def performAnalysis(
             ligaSnapshot = ligaSnapshots[0]
             ligaLengthRef = ligaSnapshot['value']
         
-        for i in range(N):
+        meshNames = [segIns1, segIns2]
+        frames = range(N)
+        saveScene = False
+        sceneOutputDir = None
+        
+        if 'args' in defLiga:
+            extraArgs = defLiga['args']
+            extraMeshNames = extraArgs['extraMeshes']
+            meshNames.extend(extraMeshNames)
+            if 'frames' in extraArgs:
+                frames = extraArgs['frames'] 
+            if 'algoArgs' in extraArgs:
+                algoArgs = extraArgs['algoArgs']
+            if 'saveScene' in extraArgs:
+                saveScene = extraArgs['saveScene']
+                sceneOutputDir = extraArgs['sceneOutputDir']
 
-            ligaPath = np.array((p1[i,:], p2[i,:]))
-    
-            if method == 'shortest_via_edge':
-    
-                splineParams = liga.reposeSpline(splinesLocParams[spline], RTe[i,...])
-                dummy, ligaPath = liga.ligamentPathBlankevoort1991(ligaPath, splineParams)
-    
-            ligaLength = np.linalg.norm(np.diff(ligaPath, axis=0), axis=1).sum()
+        meshNames = list(set(meshNames))
+        if 'global' in meshNames:
+             meshNames.remove('global')
+        meshes_ = {m: meshes[m] for m in meshNames}
+        
+        for i in range(N):
             
-            if ligaLengthRef is not None:
-                ligaStrain = 100. * (1.*ligaLength - ligaLengthRef) / ligaLengthRef
+            if i in frames:
+
+                ligaPath = np.array((p1[i,:], p2[i,:]))
+                
+                meshesReposed = [vtkh.reposeVTKData(meshes_[m], RTs[m][i,...]) for m in meshNames]
+        
+                if method == 'shortest_via_edge':
+        
+                    splineParams = liga.reposeSpline(splinesLocParams[spline], RTe[i,...])
+                    dummy, ligaPath = liga.ligamentPathBlankevoort1991(ligaPath, splineParams)
+                    
+                elif method == 'marai_2024':
+                    
+                    dummy, ligaPath = liga.ligamentPathMonari2021(ligaPath, meshesReposed[0], meshesReposed[1], **algoArgs)
+                    
+                elif method == 'monari_2021':
+                    
+                    for j in range(ligaPath.shape[0]):                                
+                        ligaPath[j,:] = vtkh.externalizePointFromMeshes(ligaPath[j,:], meshesReposed, factor=0.5)
+                    
+                    ligaPath = liga.ligamentPathMonari2021(ligaPath, meshesReposed, **algoArgs)
+                    
+                if saveScene:
+                    
+                    print('saving scene for frame %d for ligament %s... ' % (i, ligaName))
+                    
+                    actors = []
+                    actorNames = []
+                    
+                    for lp in ligaPath:
+                        actors.append(vtkh.createVTKActor(vtkh.createSphereVTKData(lp, 1)))
+                        actorNames.append('p')
+                    
+                    actors.extend([vtkh.createVTKActor(meshReposed) for meshReposed in meshesReposed])
+                    actorNames.extend(meshNames)
+                    
+                    vtkLigaPath = vtkh.createLineVTKData(ligaPath)
+                    actors.append(vtkh.createVTKActor(vtkLigaPath, color=[255,0,0]))
+                    actorNames.append(ligaName)
+                    
+                    scene = vtkh.createScene(actors)
+                    vtkh.exportScene(scene, sceneOutputDir + ('/scene_%05d' % i), ext='vtm', names=actorNames)
+        
+                ligaLength = np.linalg.norm(np.diff(ligaPath, axis=0), axis=1).sum()
+                
+                if ligaLengthRef is not None:
+                    ligaStrain = 100. * (1.*ligaLength - ligaLengthRef) / ligaLengthRef
+                else:
+                    ligaStrain = np.nan
+                    
             else:
+                
+                ligaPath = np.nan * np.ones((0,3))
+                ligaLength = np.nan
                 ligaStrain = np.nan
 
             ligaPaths[name][i] = ligaPath
             ligaLengths[name][i] = ligaLength
             ligaStrains[name][i] = ligaStrain
         
+        ls = '-'
+        ms = None
+        hasNans = np.isnan(ligaLengths[name]).any()
+        if hasNans:
+            ls = '.'
+            ms = 2.0
         plt.subplot(cr,cc,5)
-        plt.plot(t, ligaLengths[name], label=name)
+        plt.plot(t, ligaLengths[name], ls, ms=ms, label=name)
         plt.legend(loc='upper right', fontsize=7)
         plt.title('Ligament lengths (mm)')
         plt.subplot(cr,cc,6)
-        plt.plot(t, ligaStrains[name], label=name)
+        plt.plot(t, ligaStrains[name], ls, ms=ms, label=name)
         plt.legend(loc='upper right', fontsize=7)
         plt.title('Ligament strains (%)') 
 
@@ -361,7 +437,6 @@ def performAnalysis(
         lineName1 = defInterlineAngle['line1']
         lineName2 = defInterlineAngle['line2']
         
-#        angles[name] = [None] * N
         angles[name] = np.nan * np.zeros((N,))
         
         for i in range(N):
